@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: CmdExec.h,v 1.51 2004/05/28 11:48:11 lav Exp $ */
+/* $Id: CmdExec.h,v 1.52 2004/10/18 07:10:48 lav Exp $ */
 
 #ifndef CMDEXEC_H
 #define CMDEXEC_H
@@ -149,6 +149,9 @@ public:
    static void unquote(char *buf,const char *str);
    static char *unquote(const char *str);
    static bool needs_quotation(const char *buf);
+   static bool quotable(char c,char in_quotes);
+   static bool is_space(char c) { return c==' ' || c=='\t'; }
+   static bool is_quote(char c) { return c=='"' || c=='\''; }
    void FeedQuoted(const char *c);
    void AtExit();
    void EmptyCmds();
