@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileCopy.cc,v 1.104 2004/09/16 19:17:03 lav Exp $ */
+/* $Id: FileCopy.cc,v 1.106 2004/10/20 14:20:54 lav Exp $ */
 
 /* FileCopyPeer behaviour:
     1) when suspended, does nothing
@@ -663,6 +663,16 @@ bool FileCopyPeer::Done()
    return false;
 }
 
+void FileCopyPeer::Seek(off_t offs)
+{
+   seek_pos=offs;
+   if(mode==PUT)
+      pos-=in_buffer;
+   Empty();
+   eof=false;
+   broken=false;
+}
+
 FileCopyPeer::FileCopyPeer(dir_t m) : IOBuffer(m)
 {
    want_size=false;
