@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileCopyFtp.cc,v 1.17.2.1 2003/11/26 08:13:26 lav Exp $ */
+/* $Id: FileCopyFtp.cc,v 1.21 2004/05/31 13:27:36 lav Exp $ */
 
 #include <config.h>
 
@@ -29,6 +29,11 @@
 
 #define super FileCopy
 
+#if !USE_SSL
+#define protect false
+#define passive_ssl_connect true
+#endif
+
 void FileCopyFtp::Close()
 {
    ftp_src->Close();
@@ -50,7 +55,8 @@ int FileCopyFtp::Do()
       if(ftp_dst->IsClosed())
       {
 	 ((FileCopyPeerFA*)put)->OpenSession();
-	 ftp_dst->SetCopyMode(Ftp::COPY_DEST,!passive_source,dst_retries,dst_try_time);
+	 ftp_dst->SetCopyMode(Ftp::COPY_DEST,!passive_source,protect,
+	       passive_source^passive_ssl_connect,dst_retries,dst_try_time);
 	 m=MOVED;
       }
    }
@@ -65,15 +71,18 @@ int FileCopyFtp::Do()
    if(ftp_src->IsClosed())
    {
       ((FileCopyPeerFA*)get)->OpenSession();
-      ftp_src->SetCopyMode(Ftp::COPY_SOURCE,passive_source,src_retries,src_try_time);
+      ftp_src->SetCopyMode(Ftp::COPY_SOURCE,passive_source,protect,
+	    !passive_source^passive_ssl_connect,src_retries,src_try_time);
       m=MOVED;
    }
    if(ftp_dst->IsClosed())
    {
       ((FileCopyPeerFA*)put)->OpenSession();
-      ftp_dst->SetCopyMode(Ftp::COPY_DEST,!passive_source,dst_retries,dst_try_time);
+      ftp_dst->SetCopyMode(Ftp::COPY_DEST,!passive_source,protect,
+	    passive_source^passive_ssl_connect,dst_retries,dst_try_time);
       m=MOVED;
    }
+
    // check for errors
    if(ftp_src->CopyFailed() || ftp_dst->CopyFailed())
    {
@@ -82,37 +91,53 @@ int FileCopyFtp::Do()
 	 get->CannotSeek(put->GetSize());
 	 put->CannotSeek(put->GetSize());
       }
-      if((passive_source !=  ftp_src->IsCopyPassive())
-      || (passive_source != !ftp_dst->IsCopyPassive()))
+      else if(passive_source==orig_passive_source)
       {
 	 passive_source=!passive_source;
-	 if(passive_source==orig_passive_source)
+	 Log::global->Write(0,_("**** FXP: trying to reverse ftp:fxp-passive-source\n"));
+      }
+#if USE_SSL
+      else if(passive_ssl_connect==orig_passive_ssl_connect)
+      {
+	 passive_ssl_connect=!passive_ssl_connect;
+	 passive_source=orig_passive_source;
+	 Log::global->Write(0,_("**** FXP: trying to reverse ftp:fxp-passive-sscn\n"));
+      }
+      else if(protect
+      && !ResMgr::QueryBool("ftp:ssl-force",ftp_src->GetHostName())
+      && !ResMgr::QueryBool("ftp:ssl-force",ftp_dst->GetHostName()))
+      {
+	 passive_source=orig_passive_source;
+	 protect=false;
+	 Log::global->Write(0,_("**** FXP: trying to reverse ftp:ssl-protect-fxp\n"));
+      }
+#endif // USE_SSL
+      else
+      {
+	 // both ways failed. Fall back to normal copying.
+	 Log::global->Write(0,_("**** FXP: giving up, reverting to plain copy\n"));
+	 Close();
+	 disable_fxp=true;
+	 ((FileCopyPeerFA*)get)->SetFXP(false);
+	 ((FileCopyPeerFA*)put)->SetFXP(false);
+
+	 if(ResMgr::QueryBool("ftp:fxp-force",ftp_src->GetHostName())
+	 || ResMgr::QueryBool("ftp:fxp-force",ftp_dst->GetHostName()))
 	 {
-	    // both ways failed. Fall back to normal copying.
-	    Log::global->Write(0,_("**** FXP: giving up, reverting to plain copy\n"));
-	    Close();
-	    disable_fxp=true;
-	    ((FileCopyPeerFA*)get)->SetFXP(false);
-	    ((FileCopyPeerFA*)put)->SetFXP(false);
-
-	    if(ResMgr::QueryBool("ftp:fxp-force",ftp_src->GetHostName())
-	    || ResMgr::QueryBool("ftp:fxp-force",ftp_dst->GetHostName()))
-	    {
-	       SetError(_("ftp:fxp-force is set but FXP is not available"));
-	       return MOVED;
-	    }
-
-	    off_t pos=put->GetRealPos();
-	    if(!get->CanSeek(pos) || !put->CanSeek(pos))
-	       pos=0;
-	    get->Seek(pos);
-	    put->Seek(pos);
-	    RateReset();
+	    SetError(_("ftp:fxp-force is set but FXP is not available"));
 	    return MOVED;
 	 }
-	 Log::global->Write(0,_("**** FXP: trying to reverse ftp:fxp-passive-source\n"));
+
+	 off_t pos=put->GetRealPos();
+	 if(!get->CanSeek(pos) || !put->CanSeek(pos))
+	    pos=0;
+	 get->Seek(pos);
+	 put->Seek(pos);
 	 RateReset();
+	 return MOVED;
       }
+      RateReset();
+
       src_retries=ftp_src->GetRetries();
       dst_retries=ftp_dst->GetRetries();
       src_try_time=ftp_src->GetTryTime();
@@ -183,6 +208,10 @@ void FileCopyFtp::Init()
    src_retries=dst_retries=0;
    src_try_time=dst_try_time=0;
    disable_fxp=false;
+#if USE_SSL
+   protect=false;
+   orig_passive_ssl_connect=passive_ssl_connect=true;
+#endif
 }
 
 FileCopyFtp::~FileCopyFtp()
@@ -207,6 +236,14 @@ FileCopyFtp::FileCopyFtp(FileCopyPeer *s,FileCopyPeer *d,bool c,bool rp)
    else if(!ftp_src->IsPassive() && ftp_dst->IsPassive())
       passive_source=false;
    orig_passive_source=passive_source;
+
+#if USE_SSL
+   if(ResMgr::QueryBool("ftp:ssl-protect-fxp",ftp_src->GetHostName())
+   || ResMgr::QueryBool("ftp:ssl-protect-fxp",ftp_dst->GetHostName()))
+      protect=true;
+   passive_ssl_connect=ResMgr::QueryBool("ftp:fxp-passive-sscn",0);
+   orig_passive_ssl_connect=passive_ssl_connect;
+#endif
 }
 
 FileCopy *FileCopyFtp::New(FileCopyPeer *s,FileCopyPeer *d,bool c)
