@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileSet.cc,v 1.42 2004/04/22 06:32:54 lav Exp $ */
+/* $Id: FileSet.cc,v 1.43 2004/11/13 12:28:00 lav Exp $ */
 
 #include <config.h>
 #include "FileSet.h"
@@ -551,7 +551,7 @@ void FileSet::LocalChmod(const char *dir,mode_t mask)
 	 struct stat st;
 	 mode_t new_mode=file->mode&~mask;
 
-	 if(stat(local_name,&st)!=-1 && st.st_mode!=new_mode)
+	 if(stat(local_name,&st)!=-1 && (st.st_mode&07777)!=new_mode)
 	    chmod(local_name,new_mode);
       }
    }
