@@ -330,6 +330,7 @@ void Fish::Disconnect()
    if(mode==STORE)
       SetError(STORE_FAILED,0);
    received_greeting=false;
+   password_sent=0;
 }
 
 void Fish::EmptyPathQueue()
@@ -356,6 +357,7 @@ void Fish::Init()
    path_queue=0;
    path_queue_len=0;
    received_greeting=false;
+   password_sent=0;
 }
 
 Fish::Fish()
@@ -443,9 +445,16 @@ const char *Fish::shell_encode(const char *string)
    char *r;
    const char *s;
 
-   result = (char*)xrealloc (result, 2 * strlen (string) + 1);
+   result = (char*)xrealloc (result, 2 + 2 * strlen (string) + 1);
 
-   for (r = result, s = string; s && (c = *s); s++)
+   r = result;
+   if(string[0]=='-' || string[0]=='~')
+   {
+      *r++='.';
+      *r++='/';
+   }
+
+   for (s = string; s && (c = *s); s++)
    {
       switch (c)
       {
@@ -459,14 +468,10 @@ const char *Fish::shell_encode(const char *string)
       case '"': case '\\':		/* quoting chars */
       case '|': case '&': case ';':		/* shell metacharacters */
       case '<': case '>':
+      case '#':				/* comment char */
 	 *r++ = '\\';
 	 *r++ = c;
 	 break;
-      case '~':				/* tilde expansion */
-      case '#':				/* comment char */
-	 if (s == string)
-	    *r++ = '\\';
-	 /* FALLTHROUGH */
       default:
 	 *r++ = c;
 	 break;
@@ -527,12 +532,24 @@ void Fish::SendMethod()
       real_pos=0;
       break;
    case RETRIEVE:
-      Send("#RETR %s\n"
+      if(pos>0)
+      {
+	 Send("#RETRP %lld %s\n"
+	      "ls -lLd %s; "
+	      "echo '### 100'; "
+	      "cat %s|(dd ibs=1 count=%lld of=/dev/null 2>/dev/null;cat); "
+	      "echo '### 200'\n",
+	    (long long)pos,e,e,e,(long long)pos);
+      }
+      else
+      {
+	 Send("#RETR %s\n"
 	   "ls -lLd %s; "
 	   "echo '### 100'; cat %s; echo '### 200'\n",e,e,e);
+      }
       PushExpect(EXPECT_RETR_INFO);
       PushExpect(EXPECT_RETR);
-      real_pos=0;
+      real_pos=pos;
       break;
    case STORE:
       if(entity_size<0)
@@ -540,7 +557,7 @@ void Fish::SendMethod()
 	 SetError(NO_FILE,"Have to know file size before upload");
 	 break;
       }
-      // dd pays attansion to read boundaries and reads wrong number
+      // dd pays attension to read boundaries and reads wrong number
       // of bytes when ibs>1. Have to use the inefficient ibs=1.
       if(entity_size>0)
       {
@@ -656,12 +673,18 @@ int Fish::HandleReplies()
 	 {
 	    if(!pass)
 	    {
-	       SetError(LOGIN_FAILED,"Password required");
+	       SetError(LOGIN_FAILED,_("Password required"));
+	       return MOVED;
+	    }
+	    if(password_sent>0)
+	    {
+	       SetError(LOGIN_FAILED,_("Login incorrect"));
 	       return MOVED;
 	    }
 	    recv_buf->Put("XXXX");
 	    send_buf->Put(pass);
 	    send_buf->Put("\n");
+	    password_sent++;
 	    return m;
 	 }
 	 if(s>=y_len && !strncasecmp(b+s-y_len,y,y_len))
@@ -928,15 +951,12 @@ int Fish::Read(void *buf,int size)
       }
       if(size1==0)
 	 return DO_AGAIN;
-      if(entity_size>=0 && pos>=entity_size)
+      if(entity_size!=NO_SIZE && real_pos<entity_size)
       {
-	 DebugPrint("---- ",_("Received all (total)"));
-	 state=WAITING;
-	 return 0;
+	 if(real_pos+size1>entity_size)
+	    size1=entity_size-real_pos;
       }
-      if(entity_size>=0 && real_pos+size1>entity_size)
-	 size1=entity_size-real_pos;
-      if(entity_size==NO_SIZE)
+      else
       {
 	 const char *end=memstr(buf1,size1,"### ");
 	 if(end)
@@ -986,7 +1006,7 @@ int Fish::Read(void *buf,int size)
       pos+=size;
       real_pos+=size;
       rate_limit->BytesGot(size);
-      retries=0;
+      TrySuccess();
       return size;
    }
    return DO_AGAIN;
@@ -1024,7 +1044,7 @@ int Fish::Write(const void *buf,int size)
    if(size<=0)
       return 0;
    send_buf->Put((char*)buf,size);
-   retries=0;
+   TrySuccess();
    rate_limit->BytesPut(size);
    pos+=size;
    real_pos+=size;
