@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Http.cc,v 1.173 2004/08/23 14:39:36 lav Exp $ */
+/* $Id: Http.cc,v 1.179 2004/11/16 10:14:41 lav Exp $ */
 
 #include <config.h>
 #include "trio.h"
@@ -55,8 +55,6 @@ CDECL char *strptime(const char *buf, const char *format, struct tm *tm);
 #define HTTPS_DEFAULT_PORT	 "443"
 
 static time_t http_atotm (const char *time_string);
-static int  base64_length (int len);
-static void base64_encode (const char *s, char *store, int length);
 
 /* Some status code validation macros: */
 #define H_20X(x)        (((x) >= 200) && ((x) < 300))
@@ -207,7 +205,7 @@ void Http::Close()
    if(mode==CLOSED)
       return;
    if(recv_buf)
-      recv_buf->Do();	// try to read any remaining data
+      Roll(recv_buf);	// try to read any remaining data
    if(sock!=-1 && keep_alive && (keep_alive_max>0 || keep_alive_max==-1)
    && mode!=STORE && !recv_buf->Eof() && (state==RECEIVING_BODY || state==DONE))
    {
@@ -383,6 +381,8 @@ void Http::SendCacheControl()
 {
    const char *cc_setting=Query("cache-control",hostname);
    const char *cc_no_cache=(no_cache || no_cache_this)?"no-cache":0;
+   if(!*cc_setting)
+      cc_setting=0;
    if(!cc_setting && !cc_no_cache)
       return;
    int cc_no_cache_len=xstrlen(cc_no_cache);
@@ -400,10 +400,11 @@ void Http::SendCacheControl()
    if(cc_setting)
    {
       if(cc[0])
-	 strcat(cc," ");
+	 strcat(cc,",");
       strcat(cc,cc_setting);
    }
-   Send("Cache-Control: %s\r\n",cc);
+   if(*cc)
+      Send("Cache-Control: %s\r\n",cc);
 }
 
 bool Http::ModeSupported()
@@ -567,6 +568,7 @@ void Http::SendRequest(const char *connection,const char *f)
       {
 	 SendMethod("PUT",efile);   // hope it would work
 	 Send("Content-length: 0\r\n");
+	 pos=entity_size=0;
       }
       break;
 
@@ -632,7 +634,7 @@ void Http::HandleHeaderLine(const char *name,const char *value)
       if(1!=sscanf(value,"%lld",&bs))
 	 return;
       body_size=bs;
-      if(pos==0 && mode!=STORE)
+      if(pos==0 && mode!=STORE && mode!=MAKE_DIR)
 	 entity_size=body_size;
       if(pos==0 && opt_size && H_20X(status_code))
 	 *opt_size=body_size;
@@ -641,7 +643,7 @@ void Http::HandleHeaderLine(const char *name,const char *value)
       {
 	 array_for_info[array_ptr].size=body_size;
 	 array_for_info[array_ptr].get_size=false;
-	 retries=0;
+	 TrySuccess();
       }
       return;
    }
@@ -663,7 +665,7 @@ void Http::HandleHeaderLine(const char *name,const char *value)
 	 last=fsize-first-1;
       if(body_size<0)
 	 body_size=last-first+1;
-      if(mode!=STORE)
+      if(mode!=STORE && mode!=MAKE_DIR)
 	 entity_size=fsize;
       if(opt_size && H_20X(status_code))
 	 *opt_size=fsize;
@@ -679,7 +681,7 @@ void Http::HandleHeaderLine(const char *name,const char *value)
       {
 	 array_for_info[array_ptr].time=t;
 	 array_for_info[array_ptr].get_time=false;
-	 retries=0;
+	 TrySuccess();
       }
       return;
    }
@@ -1193,7 +1195,7 @@ int Http::Do()
 		  }
 		  return MOVED;
 	       }
-	       else if(mode==STORE)
+	       else if(mode==STORE || mode==MAKE_DIR)
 	       {
 		  if((sent_eot || pos==entity_size) && H_20X(status_code))
 		  {
@@ -1284,7 +1286,7 @@ int Http::Do()
 		  }
 
 		  if(mode==ARRAY_INFO)
-		     retries=0;
+		     TrySuccess();
 
 		  return MOVED;
 	       }
@@ -1400,7 +1402,7 @@ int Http::Do()
       rate_limit=new RateLimit(hostname);
       if(real_pos<0) // assume Range: did not work
       {
-	 if(mode!=STORE && body_size>=0)
+	 if(mode!=STORE && mode!=MAKE_DIR && body_size>=0)
 	 {
 	    entity_size=body_size;
 	    if(opt_size && H_20X(status_code))
@@ -1631,7 +1633,7 @@ int Http::Read(void *buf,int size)
       if(chunked)
 	 chunk_pos+=size;
       rate_limit->BytesGot(size);
-      retries=0;
+      TrySuccess();
       return size;
    }
    return DO_AGAIN;
@@ -1692,7 +1694,7 @@ int Http::Write(const void *buf,int size)
    send_buf->Put((const char*)buf,size);
 
    if(retries>0 && send_buf->GetPos()-send_buf->Size()>Buffered()+0x1000)
-      retries=0;
+      TrySuccess();
    rate_limit->BytesPut(size);
    pos+=size;
    real_pos+=size;
@@ -2220,51 +2222,6 @@ http_atotm (const char *time_string)
   return -1;
 }
 
-/* How many bytes it will take to store LEN bytes in base64.  */
-static int
-base64_length(int len)
-{
-  return (4 * (((len) + 2) / 3));
-}
-
-/* Encode the string S of length LENGTH to base64 format and place it
-   to STORE.  STORE will be 0-terminated, and must point to a writable
-   buffer of at least 1+BASE64_LENGTH(length) bytes.  */
-static void
-base64_encode (const char *s, char *store, int length)
-{
-  /* Conversion table.  */
-  static const char tbl[64] = {
-    'A','B','C','D','E','F','G','H',
-    'I','J','K','L','M','N','O','P',
-    'Q','R','S','T','U','V','W','X',
-    'Y','Z','a','b','c','d','e','f',
-    'g','h','i','j','k','l','m','n',
-    'o','p','q','r','s','t','u','v',
-    'w','x','y','z','0','1','2','3',
-    '4','5','6','7','8','9','+','/'
-  };
-  int i;
-  unsigned char *p = (unsigned char *)store;
-
-  /* Transform the 3x8 bits to 4x6 bits, as required by base64.  */
-  for (i = 0; i < length; i += 3)
-    {
-      *p++ = tbl[s[0] >> 2];
-      *p++ = tbl[((s[0] & 3) << 4) + (s[1] >> 4)];
-      *p++ = tbl[((s[1] & 0xf) << 2) + (s[2] >> 6)];
-      *p++ = tbl[s[2] & 0x3f];
-      s += 3;
-    }
-  /* Pad the result if necessary...  */
-  if (i == length + 1)
-    *(p - 1) = '=';
-  else if (i == length + 2)
-    *(p - 1) = *(p - 2) = '=';
-  /* ...and zero-terminate it.  */
-  *p = '\0';
-}
-
 
 #include "modconfig.h"
 #ifdef MODULE_PROTO_HTTP
