@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: HttpDir.cc,v 1.73.2.4 2003/12/15 07:45:19 lav Exp $ */
+/* $Id: HttpDir.cc,v 1.78 2003/12/15 06:56:08 lav Exp $ */
 
 #include <config.h>
 #include "ascii_ctype.h"
@@ -1201,7 +1201,7 @@ int HttpDirList::Do()
       if(use_cache && LsCache::Find(session,curr,mode,
 				    &cache_buffer,&cache_buffer_size))
       {
-	 ubuf=new Buffer();
+	 ubuf=new IOBuffer(IOBuffer::GET);
 	 ubuf->Put(cache_buffer,cache_buffer_size);
 	 ubuf->PutEOF();
       }
