@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: MirrorJob.cc,v 1.109 2004/04/22 06:32:55 lav Exp $ */
+/* $Id: MirrorJob.cc,v 1.116 2004/12/17 06:51:47 lav Exp $ */
 
 #include <config.h>
 
@@ -255,10 +255,10 @@ void  MirrorJob::HandleFile(FileInfo *file)
 	       args.Append("-c");
 	    if(remove_target)
 	       args.Append("-e");
+	    args.Append("-O");
+	    args.Append(target_is_local?target_dir
+			:target_session->GetConnectURL());
 	    args.Append(source_session->GetFileURL(file->name));
-	    args.Append("-o");
-	    args.Append(target_is_local?dir_file(target_dir,file->name)
-			:target_session->GetFileURL(file->name));
 	    char *cmd=args.CombineQuoted();
 	    fprintf(script,"%s\n",cmd);
 	    xfree(cmd);
@@ -380,6 +380,9 @@ void  MirrorJob::HandleFile(FileInfo *file)
       }
       case(FileInfo::SYMLINK):
       {
+	 if(flags&NO_SYMLINKS)
+	    goto skip;
+
 	 if(!target_is_local)
 	 {
 	    // can't create symlink remotely (FIXME?)
@@ -713,18 +716,17 @@ int   MirrorJob::Do()
       to_rm->rewind();
       to_rm_mismatched->Count(&stats.del_dirs,&stats.del_files,&stats.del_symlinks,&stats.del_files);
       to_rm_mismatched->rewind();
+      transfer_count-=root_transfer_count; // leave room for transfers.
       state=TARGET_REMOVE_OLD_FIRST;
       return MOVED;
 
    pre_WAITING_FOR_TRANSFER:
       to_transfer->rewind();
-      transfer_count-=root_transfer_count; // leave room for transfers.
       state=WAITING_FOR_TRANSFER;
       m=MOVED;
       /*fallthrough*/
    case(WAITING_FOR_TRANSFER):
-      j=FindDoneAwaitedJob();
-      if(j)
+      while(j=FindDoneAwaitedJob())
       {
 	 if(j->ExitCode()!=0)
 	    stats.error_count++;
@@ -756,8 +758,7 @@ int   MirrorJob::Do()
       /*fallthrough*/
    case(TARGET_REMOVE_OLD):
    case(TARGET_REMOVE_OLD_FIRST):
-      j=FindDoneAwaitedJob();
-      if(j)
+      while(j=FindDoneAwaitedJob())
       {
 	 RemoveWaiting(j);
 	 Delete(j);
@@ -778,7 +779,13 @@ int   MirrorJob::Do()
 	    to_rm->next();
 	 }
 	 if(!file)
-	    break;
+	 {
+	    if(waiting_num>0)
+	       break;
+	    if(state==TARGET_REMOVE_OLD)
+	       goto pre_TARGET_CHMOD;
+	    goto pre_WAITING_FOR_TRANSFER;
+	 }
 	 if(!(flags&DELETE))
 	 {
 	    if(flags&REPORT_NOT_DELETED)
@@ -826,12 +833,6 @@ int   MirrorJob::Do()
 		     dir_file(target_relative_dir,file->name));
 	 }
       }
-      if(waiting_num==0)
-      {
-	 if(state==TARGET_REMOVE_OLD)
-	    goto pre_TARGET_CHMOD;
-	 goto pre_WAITING_FOR_TRANSFER;
-      }
       break;
 
    pre_TARGET_CHMOD:
@@ -843,8 +844,7 @@ int   MirrorJob::Do()
       m=MOVED;
       /*fallthrough*/
    case(TARGET_CHMOD):
-      j=FindDoneAwaitedJob();
-      if(j)
+      while(j=FindDoneAwaitedJob())
       {
 	 RemoveWaiting(j);
 	 Delete(j);
@@ -871,6 +871,10 @@ int   MirrorJob::Do()
 	    if((st.st_mode&07777)==(file->mode&~mode_mask))
 	       continue;
 	 }
+	 FileInfo *target=target_set->FindByName(file->name);
+	 if(target && target->filetype==file->DIRECTORY && file->filetype==file->DIRECTORY
+	 && target->mode==(file->mode&~mode_mask) && (target->mode&0200))
+	    continue;
 	 if(script)
 	 {
 	    ArgV args("chmod");
@@ -914,8 +918,7 @@ int   MirrorJob::Do()
       m=MOVED;
       /*fallthrough*/
    case(FINISHING):
-      j=FindDoneAwaitedJob();
-      if(j)
+      while(j=FindDoneAwaitedJob())
       {
 	 RemoveWaiting(j);
 	 Delete(j);
@@ -926,11 +929,13 @@ int   MirrorJob::Do()
 	 break;
 
       transfer_count++; // parent mirror will decrement it.
+      if(parent_mirror)
+	 parent_mirror->stats.Add(stats);
       state=DONE;
       m=MOVED;
       /*fallthrough*/
    case(DONE):
-      break;
+      return m;
    }
    // give direct parent priority over grand-parents.
    if(transfer_count<parallel && parent_mirror)
@@ -1024,8 +1029,6 @@ MirrorJob::~MirrorJob()
       delete exclude;
    if(script && script_needs_closing)
       fclose(script);
-   if(parent_mirror)
-      parent_mirror->stats.Add(stats);
 }
 
 void MirrorJob::va_Report(const char *fmt,va_list v)
@@ -1185,6 +1188,7 @@ CMD(mirror)
       {"dry-run",   optional_argument,0,256+'S'},
       {"delete-first",no_argument,0,256+'e'},
       {"use-pget-n",optional_argument,0,256+'p'},
+      {"no-symlinks",no_argument,0,256+'y'},
       {0}
    };
 
@@ -1335,6 +1339,9 @@ CMD(mirror)
       case(256+'e'):
 	 flags|=MirrorJob::REMOVE_FIRST|MirrorJob::DELETE;
 	 break;
+      case(256+'y'):
+	 flags|=MirrorJob::NO_SYMLINKS;
+	 break;
       case('?'):
 	 eprintf(_("Try `help %s' for more information.\n"),args->a0());
       no_job:
