@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: MirrorJob.h,v 1.39 2004/02/17 10:45:57 lav Exp $ */
+/* $Id: MirrorJob.h,v 1.40 2004/10/28 06:23:34 lav Exp $ */
 
 #ifndef MIRRORJOB_H
 #define MIRRORJOB_H
@@ -139,7 +139,8 @@ public:
       ALLOW_CHOWN=1<<9,
       IGNORE_TIME=1<<10,
       REMOVE_FIRST=1<<11,
-      IGNORE_SIZE=1<<12
+      IGNORE_SIZE=1<<12,
+      NO_SYMLINKS=1<<13
    };
 
    void SetFlags(int f,int v)
