@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: NetAccess.cc,v 1.70 2004/08/03 11:43:23 lav Exp $ */
+/* $Id: NetAccess.cc,v 1.74 2004/12/01 12:17:43 lav Exp $ */
 
 #include <config.h>
 
@@ -53,6 +53,7 @@ void NetAccess::Init()
    idle_start=now;
    max_retries=0;
    max_persist_retries=0;
+   retries=0;
    persist_retries=0;
    socket_buffer=0;
    socket_maxseg=0;
@@ -267,6 +268,7 @@ static void test_TIOCOUTQ()
    int sock=socket(PF_INET,SOCK_STREAM,IPPROTO_TCP);
    if(sock==-1)
       return;
+   TIOCOUTQ_tested=true;
    int avail=-1;
    socklen_t len=sizeof(avail);
    if(getsockopt(sock,SOL_SOCKET,SO_SNDBUF,(char*)&avail,&len)==-1)
@@ -564,25 +566,6 @@ void RateLimit::ReconfigTotal()
    total_reconfig_needed = false;
 }
 
-long NetAccess::ReconnectInterval()
-{
-   // cyclic exponential growth.
-   float interval = reconnect_interval;
-   if(reconnect_interval_multiplier>1
-   && reconnect_interval_max>=reconnect_interval*reconnect_interval_multiplier
-   && retries>0)
-   {
-      int modval = (int)(log((float)reconnect_interval_max/reconnect_interval)
-                              / log(reconnect_interval_multiplier) + 1.999);
-
-      interval *= pow(reconnect_interval_multiplier, (retries-1)%modval);
-
-      if( interval > reconnect_interval_max )
-         interval = reconnect_interval_max;
-   }
-   return long(interval+.5);
-}
-
 bool NetAccess::ReconnectAllowed()
 {
    if(max_retries>0 && retries>=max_retries)
@@ -591,10 +574,9 @@ bool NetAccess::ReconnectAllowed()
       return false;
    if(try_time==0)
       return true;
-   long interval = ReconnectInterval();
-   if(time_t(now) >= try_time+interval)
+   if(time_t(now) >= try_time+long(reconnect_interval_current))
       return true;
-   TimeoutS(interval-(time_t(now)-try_time));
+   TimeoutS(long(reconnect_interval_current)-(time_t(now)-try_time));
    return false;
 }
 
@@ -603,7 +585,7 @@ const char *NetAccess::DelayingMessage()
    static char buf[80];
    if(connection_limit>0 && connection_limit<=CountConnections())
       return _("Connection limit reached");
-   long remains=ReconnectInterval()-(time_t(now)-try_time);
+   long remains=long(reconnect_interval_current)-(time_t(now)-try_time);
    if(remains<=0)
       return "";
    sprintf(buf,"%s: %ld",_("Delaying before reconnect"),remains);
@@ -613,17 +595,29 @@ const char *NetAccess::DelayingMessage()
 
 bool NetAccess::NextTry()
 {
-   try_time=now;
-
    if(max_retries>0 && retries>=max_retries)
    {
       Fatal(_("max-retries exceeded"));
       return false;
    }
+   if(retries==0)
+      reconnect_interval_current=reconnect_interval;
+   else if(reconnect_interval_multiplier>1)
+   {
+      reconnect_interval_current*=reconnect_interval_multiplier;
+      if(reconnect_interval_current>reconnect_interval_max)
+	 reconnect_interval_current=reconnect_interval_max;
+   }
+   try_time=now;
    retries++;
-
    return true;
 }
+void NetAccess::TrySuccess()
+{
+   retries=0;
+   persist_retries=0;
+   reconnect_interval_current=reconnect_interval;
+}
 
 void NetAccess::Close()
 {
@@ -633,8 +627,7 @@ void NetAccess::Close()
       TimeoutS(idle);
    }
 
-   retries=0;
-   persist_retries=0;
+   TrySuccess();
 
    Delete(resolver);
    resolver=0;
@@ -714,8 +707,8 @@ do_again:
 	 ubuf->SetSpeedometer(new Speedometer());
 	 if(LsCache::IsEnabled())
 	    ubuf->Save(LsCache::SizeLimit());
-	 session->Do();
-	 ubuf->Do();
+	 Roll(session);
+	 Roll(ubuf);
       }
       m=MOVED;
    }
