@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: NetAccess.h,v 1.34 2004/01/28 09:59:01 lav Exp $ */
+/* $Id: NetAccess.h,v 1.35 2004/10/14 12:02:36 lav Exp $ */
 
 #ifndef NETACCESS_H
 #define NETACCESS_H
@@ -39,6 +39,7 @@ protected:
 
    int	 max_retries;
    int	 max_persist_retries;
+   int	 retries;
    int	 persist_retries;
 
    int	 idle;
@@ -48,6 +49,7 @@ protected:
    bool	 CheckTimeout();
 
    int	 reconnect_interval;
+   float reconnect_interval_current;
    float reconnect_interval_multiplier;
    int   reconnect_interval_max;
 
@@ -90,10 +92,10 @@ protected:
 
    int Resolve(const char *defp,const char *ser,const char *pr);
 
-   long ReconnectInterval();
    const char *DelayingMessage();
    bool ReconnectAllowed();
-   bool NextTry();   // returns false if max-retries exceeded.
+   bool NextTry();	// returns false if max-retries exceeded.
+   void TrySuccess();	// reset retry counters.
 
    virtual void HandleTimeout();
 
