@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Resolver.cc,v 1.58.2.1 2003/12/11 08:49:48 lav Exp $ */
+/* $Id: Resolver.cc,v 1.62 2004/09/06 08:16:51 lav Exp $ */
 
 #include <config.h>
 
@@ -56,17 +56,17 @@
 # define T_SRV 33
 #endif
 
-#if !defined(HAVE_HSTRERROR_DECL)
+#if !HAVE_DECL_HSTRERROR
 extern "C" { const char *hstrerror(int); }
 #endif
 
 #ifdef HAVE_H_ERRNO
-#ifndef HAVE_H_ERRNO_DECL
+#if !HAVE_DECL_H_ERRNO
 CDECL int h_errno;
 #endif
 #endif
 
-#if defined(HAVE_RES_SEARCH) && !defined(HAVE_RES_SEARCH_DECL)
+#if HAVE_RES_SEARCH && !HAVE_DECL_RES_SEARCH
 CDECL int res_search(const char*,int,int,unsigned char*,int);
 #endif
 
@@ -227,7 +227,7 @@ int   Resolver::Do()
       if(!buf)
       {
 	 Log::global->Format(4,"---- %s\n",_("Resolving host address..."));
-	 buf=new Buffer();
+	 buf=new IOBuffer(IOBuffer::GET);
 	 DoGethostbyname();
 	 if(deleting)
 	    return MOVED;
@@ -331,6 +331,9 @@ void Resolver::AddAddress(int family,const char *address,int len)
       }
       memcpy(&add->in.sin_addr,address,len);
       add->in.sin_port=port_number;
+#ifdef HAVE_STRUCT_SOCKADDR_SA_LEN
+      add->sa.sa_len=sizeof(add->in);
+#endif
       break;
 
 #if INET6
@@ -342,6 +345,9 @@ void Resolver::AddAddress(int family,const char *address,int len)
       }
       memcpy(&add->in6.sin6_addr,address,len);
       add->in6.sin6_port=port_number;
+#ifdef HAVE_STRUCT_SOCKADDR_SA_LEN
+      add->sa.sa_len=sizeof(add->in6);
+#endif
       break;
 #endif
 
@@ -384,6 +390,11 @@ static
 int extract_domain(const unsigned char *answer,const unsigned char *scan,int len,
 		     char *store,int store_len)
 {
+#ifdef HAVE_DN_EXPAND // newer resolver versions have dn_expand and dn_skipname
+   if(store)
+      dn_expand(answer,scan+len,scan,store,store_len);
+   return dn_skipname(scan,scan+len);
+#else // ...older don't.
    int count=1;	  // reserve space for \0
    int refs=0;
    int consumed=0;
@@ -440,11 +451,19 @@ int extract_domain(const unsigned char *answer,const unsigned char *scan,int len
    if(refs==0)
       consumed=scan-start;
    return consumed;
+#endif // DN_EXPAND
 }
 
+#ifndef NS_MAXDNAME
+# define NS_MAXDNAME 1025
+#endif
+#ifndef NS_HFIXEDSZ
+# define NS_HFIXEDSZ 12
+#endif
+
 struct SRV
 {
-   char domain[256];
+   char domain[NS_MAXDNAME];
    int port;
    int priority;
    int weight;
@@ -510,20 +529,22 @@ void Resolver::LookupSRV_RR()
    if(len>(int)sizeof(answer))
       len=sizeof(answer);
 
-   if(len<12)
+   if(len<NS_HFIXEDSZ)
       return;
 
    int question_count=(answer[4]<<8)+answer[5];
    int answer_count  =(answer[6]<<8)+answer[7];
 
    // skip header
-   unsigned char *scan=answer+12;
-   len-=12;
+   unsigned char *scan=answer+NS_HFIXEDSZ;
+   len-=NS_HFIXEDSZ;
 
    // skip questions section
    for( ; question_count>0; question_count--)
    {
       int dom_len=extract_domain(answer,scan,len,0,0);
+      if(dom_len<0)
+	 return;
       scan+=dom_len;
       len-=dom_len;
       if(len<4)
@@ -539,6 +560,11 @@ void Resolver::LookupSRV_RR()
    for( ; answer_count>0; answer_count--)
    {
       int dom_len=extract_domain(answer,scan,len,0,0);
+      if(dom_len<0)
+      {
+	 xfree(SRVs);
+	 return;
+      }
       scan+=dom_len;
       len-=dom_len;
       if(len<8)
@@ -573,6 +599,11 @@ void Resolver::LookupSRV_RR()
       len-=6;
 
       dom_len=extract_domain(answer,scan,len,t->domain,sizeof(t->domain));
+      if(dom_len<0)
+      {
+	 xfree(SRVs);
+	 return;
+      }
       scan+=dom_len;
       len-=dom_len;
 
@@ -637,6 +668,7 @@ void Resolver::LookupSRV_RR()
    }
    port_number=oldport;
 
+   xfree(SRVs);
 #endif // HAVE_RES_SEARCH
 }
 
