@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: SFtp.cc,v 1.31 2004/09/09 09:32:32 lav Exp $ */
+/* $Id: SFtp.cc,v 1.35 2004/11/26 10:14:56 lav Exp $ */
 
 #include <config.h>
 #include "SFtp.h"
@@ -345,6 +345,7 @@ void SFtp::Disconnect()
    if(mode==STORE)
       SetError(STORE_FAILED);
    received_greeting=false;
+   password_sent=0;
    protocol_version=0;
    delete send_translate; send_translate=0;
    delete recv_translate; recv_translate=0;
@@ -367,6 +368,7 @@ void SFtp::Init()
    ssh_id=0;
    eof=false;
    received_greeting=false;
+   password_sent=0;
    expect_queue_size=0;
    expect_chain=0;
    expect_chain_end=&expect_chain;
@@ -736,12 +738,18 @@ int SFtp::HandlePty()
       {
 	 if(!pass)
 	 {
-	    SetError(LOGIN_FAILED,"Password required");
+	    SetError(LOGIN_FAILED,_("Password required"));
+	    return MOVED;
+	 }
+	 if(password_sent>0)
+	 {
+	    SetError(LOGIN_FAILED,_("Login incorrect"));
 	    return MOVED;
 	 }
 	 pty_recv_buf->Put("XXXX");
 	 pty_send_buf->Put(pass);
 	 pty_send_buf->Put("\n");
+	 password_sent++;
 	 return m;
       }
       if(s>=y_len && !strncasecmp(b+s-y_len,y,y_len))
@@ -1190,7 +1198,7 @@ int SFtp::Read(void *buf,int size)
       pos+=size;
       real_pos+=size;
       rate_limit->BytesGot(size);
-      retries=0;
+      TrySuccess();
       return size;
    }
    return DO_AGAIN;
@@ -1224,7 +1232,7 @@ int SFtp::Write(const void *buf,int size)
    if(size<=0)
       return 0;
    file_buf->Put((char*)buf,size);
-   retries=0;
+   TrySuccess();
    rate_limit->BytesPut(size);
    pos+=size;
    real_pos+=size;
@@ -1274,6 +1282,10 @@ void SFtp::Suspend()
    }
    if(send_buf)
       send_buf->Suspend();
+   if(pty_send_buf)
+      pty_send_buf->Suspend();
+   if(pty_recv_buf)
+      pty_recv_buf->Suspend();
    super::Suspend();
 }
 void SFtp::Resume()
@@ -1285,6 +1297,10 @@ void SFtp::Resume()
       recv_buf->Resume();
    if(send_buf)
       send_buf->Resume();
+   if(pty_send_buf)
+      pty_send_buf->Resume();
+   if(pty_recv_buf)
+      pty_recv_buf->Resume();
 }
 
 const char *SFtp::CurrentStatus()
@@ -2023,6 +2039,8 @@ int SFtpListInfo::Do()
       if(!result && session->IsOpen())
 	 result=((SFtp*)session)->GetFileSet();
       LsCache::Add(session,"",FA::LONG_LIST, ubuf, result);
+      result->ExcludeDots();
+      result->Exclude(exclude_prefix,exclude);
       done=true;
       m=MOVED;
    }
