@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Speedometer.cc,v 1.18 2003/10/03 14:18:46 lav Exp $ */
+/* $Id: Speedometer.cc,v 1.19 2004/05/20 14:32:03 lav Exp $ */
 
 #include <config.h>
 #include <math.h>
@@ -62,6 +62,9 @@ float Speedometer::Get()
 }
 void Speedometer::Add(int b)
 {
+   if(b==0 && TimeDiff(now,last_second).MilliSeconds()<100)
+      return;
+
    // This makes Speedometer start only when first data come.
    if(rate==0)
       Reset();
