@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2001 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2004 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,27 +18,30 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: StatusLine.h,v 1.13 2002/07/23 14:03:34 lav Exp $ */
+/* $Id: StatusLine.h,v 1.17 2004/05/11 07:31:50 lav Exp $ */
 
 #ifndef STATUSLINE_H
 #define STATUSLINE_H
 
 #include "SMTask.h"
 #include "Timer.h"
+#include "StringSet.h"
 
 class StatusLine : public SMTask
 {
    int fd;
-   char shown[0x800];
+   StringSet shown;
    bool	not_term;
    Timer update_timer;
-   char to_be_shown[0x800];
+   StringSet to_be_shown;
    char def_title[0x800];
    bool update_delayed;
-   void update(char *);
+   void update(const char *const *,int);
+   void update(const char *s) { update(&s,1); }
    int LastWidth;
+   int LastHeight;
    bool next_update_title_only;
-   static const char *to_status_line, *from_status_line;
+   static const char *to_status_line, *from_status_line, *prev_line;
 
 protected:
    ~StatusLine();
@@ -46,12 +49,18 @@ protected:
 
 public:
    int GetWidth();
+   int GetHeight();
    int GetWidthDelayed() const { return LastWidth; }
+   int GetHeightDelayed() const { return LastHeight; }
    void NextUpdateTitleOnly() { next_update_title_only=true; }
    void DefaultTitle(const char *s);
+   void ShowN(const char *const* newstr,int n);
    void Show(const char *f,...) PRINTF_LIKE(2,3);
+   void Show(const StringSet &s) { ShowN(s.Set(),s.Count()); }
+   void Show(const StringSet *s) { Show(*s); }
    void WriteLine(const char *f,...) PRINTF_LIKE(2,3);
    void Clear(bool title_also=true);
+   bool CanShowNow() { return update_timer.Stopped(); }
 
    int getfd() const { return fd; }
 
