@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: buffer.h,v 1.34 2004/02/17 10:45:58 lav Exp $ */
+/* $Id: buffer.h,v 1.35 2004/08/03 11:34:38 lav Exp $ */
 
 #ifndef BUFFER_H
 #define BUFFER_H
@@ -173,7 +173,8 @@ public:
       {
 	 return(broken || Error() || (eof && (mode==GET || in_buffer==0)));
       }
-   int Do() { return STALL; }
+   virtual int Get_LL(int size) { return 0; }
+   virtual int Do();
 
    virtual FgData *GetFgData(bool) { return 0; }
    virtual const char *Status() { return ""; }
@@ -209,7 +210,6 @@ protected:
 public:
    IOBufferFDStream(FDStream *o,dir_t m,Timer *t=0)
       : IOBuffer(m) { stream=o; put_ll_timer=t; }
-   int Do();
    bool Done();
    FgData *GetFgData(bool fg);
    const char *Status() { return stream->status; }
@@ -228,7 +228,6 @@ protected:
 
 public:
    IOBufferFileAccess(FileAccess *i) : IOBuffer(GET) { session=i; }
-   int Do();
 
    void Suspend();
    void Resume();
