@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: mgetJob.cc,v 1.19 2003/03/11 13:17:05 lav Exp $ */
+/* $Id: mgetJob.cc,v 1.20 2004/02/17 10:40:59 lav Exp $ */
 
 #include <config.h>
 #include <sys/types.h>
@@ -162,7 +162,7 @@ int mgetJob::Do()
    {
    next:
       if(rg) { delete rg; rg=0; }
-      char *p=m_args->getnext();
+      const char *p=m_args->getnext();
       if(!p)
       {
 	 delete m_args;
