@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: resource.cc,v 1.108 2004/08/23 14:39:37 lav Exp $ */
+/* $Id: resource.cc,v 1.110 2004/11/19 12:58:11 lav Exp $ */
 
 #include <config.h>
 
@@ -159,6 +159,23 @@ const char *AuthArgValidate(char **s)
 
    return 0;
 }
+static
+const char *ProtValidate(char **s)
+{
+   if(!**s)
+      return 0;
+
+   for(char *i=*s; *i; i++)
+      *i=to_ascii_upper(*i);
+
+   if(strcmp(*s,"P")
+   && strcmp(*s,"C")
+   && strcmp(*s,"S")
+   && strcmp(*s,"E"))
+      return _("must be one of: C, S, E, P, or empty");
+
+   return 0;
+}
 #endif
 
 // Static array of objects is wrongly initialized by IRIX CC and Unixware c++.
@@ -201,6 +218,7 @@ static ResDecl
    ResDecl11f("ftp:ssl-protect-list",	  "yes",   ResMgr::BoolValidate,0),
    ResDecl11d("ftp:ssl-auth",		  "TLS",   AuthArgValidate,0),
    ResDecl11e("ftp:ssl-allow-anonymous",  "no",	   ResMgr::BoolValidate,0),
+   ResDecl11h("ftps:initial-prot",	  "",	   ProtValidate,0),
 #endif
    ResDecl12 ("ftp:stat-interval",	  "1",	   ResMgr::UNumberValidate,0),
    ResDecl13 ("ftp:sync-mode",		  "on",    ResMgr::BoolValidate,0),
